@@ -48,41 +48,23 @@
 #include "gdata-private.h"
 #include "gdata-types.h"
 
-typedef enum {
-	GDATA_QUERY_PARAM_Q = 1 << 0,
-	GDATA_QUERY_PARAM_CATEGORIES = 1 << 1,
-	GDATA_QUERY_PARAM_AUTHOR = 1 << 2,
-	GDATA_QUERY_PARAM_UPDATED_MIN = 1 << 3,
-	GDATA_QUERY_PARAM_UPDATED_MAX = 1 << 4,
-	GDATA_QUERY_PARAM_PUBLISHED_MIN = 1 << 5,
-	GDATA_QUERY_PARAM_PUBLISHED_MAX = 1 << 6,
-	GDATA_QUERY_PARAM_START_INDEX = 1 << 7,
-	GDATA_QUERY_PARAM_IS_STRICT = 1 << 8,
-	GDATA_QUERY_PARAM_MAX_RESULTS = 1 << 9,
-	GDATA_QUERY_PARAM_ENTRY_ID = 1 << 10,
-	GDATA_QUERY_PARAM_ALL = (1 << 11) - 1
-} GDataQueryParam;
-
 static void gdata_query_finalize (GObject *object);
 static void gdata_query_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_query_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static void get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboolean *params_started);
 
 struct _GDataQueryPrivate {
-	guint parameter_mask; /* GDataQueryParam */
-
 	/* Standard query parameters (see: http://code.google.com/apis/gdata/docs/2.0/reference.html#Queries) */
 	gchar *q;
 	gchar *categories;
 	gchar *author;
-	GTimeVal updated_min;
-	GTimeVal updated_max;
-	GTimeVal published_min;
-	GTimeVal published_max;
+	gint64 updated_min;
+	gint64 updated_max;
+	gint64 published_min;
+	gint64 published_max;
 	gint start_index;
 	gboolean is_strict;
 	gint max_results;
-	gchar *entry_id;
 
 	gchar *next_uri;
 	gchar *previous_uri;
@@ -103,12 +85,10 @@ enum {
 	PROP_START_INDEX,
 	PROP_IS_STRICT,
 	PROP_MAX_RESULTS,
-	PROP_ENTRY_ID,
 	PROP_ETAG
 };
 
 G_DEFINE_TYPE (GDataQuery, gdata_query, G_TYPE_OBJECT)
-#define GDATA_QUERY_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_QUERY, GDataQueryPrivate))
 
 static void
 gdata_query_class_init (GDataQueryClass *klass)
@@ -143,10 +123,10 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * word "Austen", use the following query: <userinput>"Elizabeth Bennet" Darcy -Austen</userinput>.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_Q,
-				g_param_spec_string ("q",
-					"Query terms", "Query terms for which to search.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("q",
+	                                                      "Query terms", "Query terms for which to search.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:categories:
@@ -160,9 +140,9 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * To do an OR between terms, use a pipe character (<literal>|</literal>). For example: <userinput>Fritz\%7CLaurie</userinput> returns
 	 * entries that match either category.
 	 *
-	 * An entry matches a specified category if the entry is in a category that has a matching term or label, as defined in the Atom specification.
-	 * (Roughly, the "term" is the internal string used by the software to identify the category, while the "label" is the human-readable string
-	 * presented to a user in a user interface.)
+	 * An entry matches a specified category if the entry is in a category that has a matching term or label, as defined in the Atom
+	 * specification. (Roughly, the "term" is the internal string used by the software to identify the category, while the "label" is the
+	 * human-readable string presented to a user in a user interface.)
 	 *
 	 * To exclude entries that match a given category, use the form <userinput>-categoryname</userinput>.
 	 *
@@ -173,10 +153,10 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * The above features can be combined. For example: <userinput>A|-{urn:google.com}B/-C</userinput> means (A OR (NOT B)) AND (NOT C).
 	 **/
 	g_object_class_install_property (gobject_class, PROP_CATEGORIES,
-				g_param_spec_string ("categories",
-					"Category string", "Category search string.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("categories",
+	                                                      "Category string", "Category search string.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:author:
@@ -184,10 +164,10 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * An entry author. The service returns entries where the author name and/or e-mail address match your query string.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_AUTHOR,
-				g_param_spec_string ("author",
-					"Author", "Author search string.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("author",
+	                                                      "Author", "Author search string.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:updated-min:
@@ -195,10 +175,10 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * Lower bound on the entry update date, inclusive.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_UPDATED_MIN,
-				g_param_spec_boxed ("updated-min",
-					"Minimum update date", "Minimum date for updates on returned entries.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("updated-min",
+	                                                     "Minimum update date", "Minimum date for updates on returned entries.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:updated-max:
@@ -206,10 +186,10 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * Upper bound on the entry update date, exclusive.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_UPDATED_MAX,
-				g_param_spec_boxed ("updated-max",
-					"Maximum update date", "Maximum date for updates on returned entries.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("updated-max",
+	                                                     "Maximum update date", "Maximum date for updates on returned entries.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:published-min:
@@ -217,10 +197,10 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * Lower bound on the entry publish date, inclusive.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_PUBLISHED_MIN,
-				g_param_spec_boxed ("published-min",
-					"Minimum publish date", "Minimum date for returned entries to be published.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("published-min",
+	                                                     "Minimum publish date", "Minimum date for returned entries to be published.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:published-max:
@@ -228,22 +208,24 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * Upper bound on the entry publish date, exclusive.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_PUBLISHED_MAX,
-				g_param_spec_boxed ("published-max",
-					"Maximum publish date", "Maximum date for returned entries to be published.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("published-max",
+	                                                     "Maximum publish date", "Maximum date for returned entries to be published.",
+	                                                     -1, G_MAXINT64, -1,
+	                                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:start-index:
 	 *
 	 * The one-based index of the first result to be retrieved. Use gdata_query_next_page() and gdata_query_previous_page() to
 	 * implement pagination, rather than manually changing #GDataQuery:start-index.
+	 *
+	 * Use <code class="literal">0</code> to not specify a start index.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_START_INDEX,
-				g_param_spec_int ("start-index",
-					"Start index", "Zero-based result start index.",
-					-1, G_MAXINT, -1,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_uint ("start-index",
+	                                                    "Start index", "One-based result start index.",
+	                                                    0, G_MAXUINT, 0,
+	                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:is-strict:
@@ -254,33 +236,24 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * Since: 0.2.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_IS_STRICT,
-				g_param_spec_boolean ("is-strict",
-					"Strict?", "Should the server be strict about the query?",
-					FALSE,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_boolean ("is-strict",
+	                                                       "Strict?", "Should the server be strict about the query?",
+	                                                       FALSE,
+	                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:max-results:
 	 *
 	 * Maximum number of results to be retrieved. Most services have a default #GDataQuery:max-results size imposed by the server; if you wish
-	 * to receive the entire feed, specify a large number such as %G_MAXINT for this property.
+	 * to receive the entire feed, specify a large number such as %G_MAXUINT for this property.
+	 *
+	 * Use <code class="literal">0</code> to not specify a maximum number of results.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_MAX_RESULTS,
-				g_param_spec_int ("max-results",
-					"Maximum number of results", "The maximum number of entries to return.",
-					-1, G_MAXINT, -1,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-	/**
-	 * GDataQuery:entry-id:
-	 *
-	 * The ID of a specific entry to be retrieved. If you specify an entry ID, you cannot specify any other parameters.
-	 **/
-	g_object_class_install_property (gobject_class, PROP_ENTRY_ID,
-				g_param_spec_string ("entry-id",
-					"Entry ID", "A specific entry ID to return.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_uint ("max-results",
+	                                                    "Maximum number of results", "The maximum number of entries to return.",
+	                                                    0, G_MAXUINT, 0,
+	                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataQuery:etag:
@@ -294,33 +267,32 @@ gdata_query_class_init (GDataQueryClass *klass)
 	 * Since: 0.2.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ETAG,
-				g_param_spec_string ("etag",
-					"ETag", "An ETag against which to check.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("etag",
+	                                                      "ETag", "An ETag against which to check.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
 gdata_query_init (GDataQuery *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_QUERY, GDataQueryPrivate);
-	self->priv->start_index = -1;
-	self->priv->max_results = -1;
+	self->priv->updated_min = -1;
+	self->priv->updated_max = -1;
+	self->priv->published_min = -1;
+	self->priv->published_max = -1;
 }
 
 static void
 gdata_query_finalize (GObject *object)
 {
-	GDataQueryPrivate *priv = GDATA_QUERY_GET_PRIVATE (object);
+	GDataQueryPrivate *priv = GDATA_QUERY (object)->priv;
 
 	g_free (priv->q);
 	g_free (priv->categories);
 	g_free (priv->author);
-	g_free (priv->entry_id);
-
 	g_free (priv->next_uri);
 	g_free (priv->previous_uri);
-
 	g_free (priv->etag);
 
 	/* Chain up to the parent class */
@@ -330,7 +302,7 @@ gdata_query_finalize (GObject *object)
 static void
 gdata_query_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
-	GDataQueryPrivate *priv = GDATA_QUERY_GET_PRIVATE (object);
+	GDataQueryPrivate *priv = GDATA_QUERY (object)->priv;
 
 	switch (property_id) {
 		case PROP_Q:
@@ -343,28 +315,25 @@ gdata_query_get_property (GObject *object, guint property_id, GValue *value, GPa
 			g_value_set_string (value, priv->author);
 			break;
 		case PROP_UPDATED_MIN:
-			g_value_set_boxed (value, &(priv->updated_min));
+			g_value_set_int64 (value, priv->updated_min);
 			break;
 		case PROP_UPDATED_MAX:
-			g_value_set_boxed (value, &(priv->updated_max));
+			g_value_set_int64 (value, priv->updated_max);
 			break;
 		case PROP_PUBLISHED_MIN:
-			g_value_set_boxed (value, &(priv->published_min));
+			g_value_set_int64 (value, priv->published_min);
 			break;
 		case PROP_PUBLISHED_MAX:
-			g_value_set_boxed (value, &(priv->published_max));
+			g_value_set_int64 (value, priv->published_max);
 			break;
 		case PROP_START_INDEX:
-			g_value_set_int (value, priv->start_index);
+			g_value_set_uint (value, priv->start_index);
 			break;
 		case PROP_IS_STRICT:
 			g_value_set_boolean (value, priv->is_strict);
 			break;
 		case PROP_MAX_RESULTS:
-			g_value_set_int (value, priv->max_results);
-			break;
-		case PROP_ENTRY_ID:
-			g_value_set_string (value, priv->entry_id);
+			g_value_set_uint (value, priv->max_results);
 			break;
 		case PROP_ETAG:
 			g_value_set_string (value, priv->etag);
@@ -392,28 +361,25 @@ gdata_query_set_property (GObject *object, guint property_id, const GValue *valu
 			gdata_query_set_author (self, g_value_get_string (value));
 			break;
 		case PROP_UPDATED_MIN:
-			gdata_query_set_updated_min (self, g_value_get_boxed (value));
+			gdata_query_set_updated_min (self, g_value_get_int64 (value));
 			break;
 		case PROP_UPDATED_MAX:
-			gdata_query_set_updated_max (self, g_value_get_boxed (value));
+			gdata_query_set_updated_max (self, g_value_get_int64 (value));
 			break;
 		case PROP_PUBLISHED_MIN:
-			gdata_query_set_published_min (self, g_value_get_boxed (value));
+			gdata_query_set_published_min (self, g_value_get_int64 (value));
 			break;
 		case PROP_PUBLISHED_MAX:
-			gdata_query_set_published_max (self, g_value_get_boxed (value));
+			gdata_query_set_published_max (self, g_value_get_int64 (value));
 			break;
 		case PROP_START_INDEX:
-			gdata_query_set_start_index (self, g_value_get_int (value));
+			gdata_query_set_start_index (self, g_value_get_uint (value));
 			break;
 		case PROP_IS_STRICT:
 			gdata_query_set_is_strict (self, g_value_get_boolean (value));
 			break;
 		case PROP_MAX_RESULTS:
-			gdata_query_set_max_results (self, g_value_get_int (value));
-			break;
-		case PROP_ENTRY_ID:
-			gdata_query_set_entry_id (self, g_value_get_string (value));
+			gdata_query_set_max_results (self, g_value_get_uint (value));
 			break;
 		case PROP_ETAG:
 			gdata_query_set_etag (self, g_value_get_string (value));
@@ -432,26 +398,12 @@ get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboo
 
 	#define APPEND_SEP g_string_append_c (query_uri, (*params_started == FALSE) ? '?' : '&'); *params_started = TRUE;
 
-	/* Check to see if any parameters have been set */
-	if ((priv->parameter_mask & GDATA_QUERY_PARAM_ALL) == 0)
-		return;
-
-	/* If we've been provided with an entry ID, only append that */
-	if (priv->entry_id != NULL) {
-		g_string_append_c (query_uri, '/');
-		g_string_append_uri_escaped (query_uri, priv->entry_id, NULL, FALSE);
-		return;
-	}
-
+	/* Categories */
 	if (priv->categories != NULL) {
 		g_string_append (query_uri, "/-/");
 		g_string_append_uri_escaped (query_uri, priv->categories, "/", FALSE);
 	}
 
-	/* If that's it, return */
-	if ((priv->parameter_mask & (GDATA_QUERY_PARAM_ALL ^ GDATA_QUERY_PARAM_ENTRY_ID ^ GDATA_QUERY_PARAM_CATEGORIES)) == 0)
-		return;
-
 	/* q param */
 	if (priv->q != NULL) {
 		APPEND_SEP
@@ -465,49 +417,49 @@ get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboo
 		g_string_append_uri_escaped (query_uri, priv->author, NULL, FALSE);
 	}
 
-	if (priv->updated_min.tv_sec != 0 || priv->updated_min.tv_usec != 0) {
+	if (priv->updated_min != -1) {
 		gchar *updated_min;
 
 		APPEND_SEP
 		g_string_append (query_uri, "updated-min=");
-		updated_min = g_time_val_to_iso8601 (&(priv->updated_min));
+		updated_min = gdata_parser_int64_to_iso8601 (priv->updated_min);
 		g_string_append (query_uri, updated_min);
 		g_free (updated_min);
 	}
 
-	if (priv->updated_max.tv_sec != 0 || priv->updated_max.tv_usec != 0) {
+	if (priv->updated_max != -1) {
 		gchar *updated_max;
 
 		APPEND_SEP
 		g_string_append (query_uri, "updated-max=");
-		updated_max = g_time_val_to_iso8601 (&(priv->updated_max));
+		updated_max = gdata_parser_int64_to_iso8601 (priv->updated_max);
 		g_string_append (query_uri, updated_max);
 		g_free (updated_max);
 	}
 
-	if (priv->published_min.tv_sec != 0 || priv->published_min.tv_usec != 0) {
+	if (priv->published_min != -1) {
 		gchar *published_min;
 
 		APPEND_SEP
 		g_string_append (query_uri, "published-min=");
-		published_min = g_time_val_to_iso8601 (&(priv->published_min));
+		published_min = gdata_parser_int64_to_iso8601 (priv->published_min);
 		g_string_append (query_uri, published_min);
 		g_free (published_min);
 	}
 
-	if (priv->published_max.tv_sec != 0 || priv->published_max.tv_usec != 0) {
+	if (priv->published_max != -1) {
 		gchar *published_max;
 
 		APPEND_SEP
 		g_string_append (query_uri, "published-max=");
-		published_max = g_time_val_to_iso8601 (&(priv->published_max));
+		published_max = gdata_parser_int64_to_iso8601 (priv->published_max);
 		g_string_append (query_uri, published_max);
 		g_free (published_max);
 	}
 
 	if (priv->start_index > 0) {
 		APPEND_SEP
-		g_string_append_printf (query_uri, "start-index=%d", priv->start_index);
+		g_string_append_printf (query_uri, "start-index=%u", priv->start_index);
 	}
 
 	if (priv->is_strict == TRUE) {
@@ -517,13 +469,13 @@ get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboo
 
 	if (priv->max_results > 0) {
 		APPEND_SEP
-		g_string_append_printf (query_uri, "max-results=%d", priv->max_results);
+		g_string_append_printf (query_uri, "max-results=%u", priv->max_results);
 	}
 }
 
 /**
  * gdata_query_new:
- * @q: a query string
+ * @q: (allow-none): a query string, or %NULL
  *
  * Creates a new #GDataQuery with its #GDataQuery:q property set to @q.
  *
@@ -537,9 +489,9 @@ gdata_query_new (const gchar *q)
 
 /**
  * gdata_query_new_with_limits:
- * @q: a query string
- * @start_index: a one-based start index for the results
- * @max_results: the maximum number of results to return
+ * @q: (allow-none): a query string, or %NULL
+ * @start_index: a one-based start index for the results, or <code class="literal">0</code>
+ * @max_results: the maximum number of results to return, or <code class="literal">0</code>
  *
  * Creates a new #GDataQuery with its #GDataQuery:q property set to @q, and the limits @start_index and @max_results
  * applied.
@@ -547,27 +499,13 @@ gdata_query_new (const gchar *q)
  * Return value: a new #GDataQuery
  **/
 GDataQuery *
-gdata_query_new_with_limits (const gchar *q, gint start_index, gint max_results)
+gdata_query_new_with_limits (const gchar *q, guint start_index, guint max_results)
 {
 	return g_object_new (GDATA_TYPE_QUERY,
-			     "q", q,
-			     "start-index", start_index,
-			     "max-results", max_results,
-			     NULL);
-}
-
-/**
- * gdata_query_new_for_id:
- * @entry_id: an entry URN ID
- *
- * Creates a new #GDataQuery to query for @entry_id.
- *
- * Return value: a new #GDataQuery
- **/
-GDataQuery *
-gdata_query_new_for_id (const gchar *entry_id)
-{
-	return g_object_new (GDATA_TYPE_QUERY, "entry-id", entry_id, NULL);
+	                     "q", q,
+	                     "start-index", start_index,
+	                     "max-results", max_results,
+	                     NULL);
 }
 
 /**
@@ -589,6 +527,9 @@ gdata_query_get_query_uri (GDataQuery *self, const gchar *feed_uri)
 	GString *query_uri;
 	gboolean params_started;
 
+	g_return_val_if_fail (GDATA_IS_QUERY (self), NULL);
+	g_return_val_if_fail (feed_uri != NULL, NULL);
+
 	/* Check to see if we're paginating first */
 	if (self->priv->use_next_uri == TRUE)
 		return g_strdup (self->priv->next_uri);
@@ -626,7 +567,7 @@ gdata_query_get_q (GDataQuery *self)
 /**
  * gdata_query_set_q:
  * @self: a #GDataQuery
- * @q: a new query string, or %NULL
+ * @q: (allow-none): a new query string, or %NULL
  *
  * Sets the #GDataQuery:q property of the #GDataQuery to the new query string, @q.
  *
@@ -639,12 +580,6 @@ gdata_query_set_q (GDataQuery *self, const gchar *q)
 
 	g_free (self->priv->q);
 	self->priv->q = g_strdup (q);
-
-	if (q == NULL)
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_Q;
-	else
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_Q;
-
 	g_object_notify (G_OBJECT (self), "q");
 
 	/* Our current ETag will no longer be relevant */
@@ -669,7 +604,7 @@ gdata_query_get_categories (GDataQuery *self)
 /**
  * gdata_query_set_categories:
  * @self: a #GDataQuery
- * @categories: the new category string, or %NULL
+ * @categories: (allow-none): the new category string, or %NULL
  *
  * Sets the #GDataQuery:categories property of the #GDataQuery to the new category string, @categories.
  *
@@ -682,12 +617,6 @@ gdata_query_set_categories (GDataQuery *self, const gchar *categories)
 
 	g_free (self->priv->categories);
 	self->priv->categories = g_strdup (categories);
-
-	if (categories == NULL)
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_CATEGORIES;
-	else
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_CATEGORIES;
-
 	g_object_notify (G_OBJECT (self), "categories");
 
 	/* Our current ETag will no longer be relevant */
@@ -712,7 +641,7 @@ gdata_query_get_author (GDataQuery *self)
 /**
  * gdata_query_set_author:
  * @self: a #GDataQuery
- * @author: the new author string, or %NULL
+ * @author: (allow-none): the new author string, or %NULL
  *
  * Sets the #GDataQuery:author property of the #GDataQuery to the new author string, @author.
  *
@@ -725,12 +654,6 @@ gdata_query_set_author (GDataQuery *self, const gchar *author)
 
 	g_free (self->priv->author);
 	self->priv->author = g_strdup (author);
-
-	if (author == NULL)
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_AUTHOR;
-	else
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_AUTHOR;
-
 	g_object_notify (G_OBJECT (self), "author");
 
 	/* Our current ETag will no longer be relevant */
@@ -740,42 +663,34 @@ gdata_query_set_author (GDataQuery *self, const gchar *author)
 /**
  * gdata_query_get_updated_min:
  * @self: a #GDataQuery
- * @updated_min: a #GTimeVal
  *
- * Gets the #GDataQuery:updated-min property and puts it in @updated_min. If the property is unset,
- * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
+ * Gets the #GDataQuery:updated-min property. If the property is unset, <code class="literal">-1</code> will be returned.
+ *
+ * Return value: the updated-min property, or <code class="literal">-1</code>
  **/
-void
-gdata_query_get_updated_min (GDataQuery *self, GTimeVal *updated_min)
+gint64
+gdata_query_get_updated_min (GDataQuery *self)
 {
-	g_return_if_fail (GDATA_IS_QUERY (self));
-	g_return_if_fail (updated_min != NULL);
-	*updated_min = self->priv->updated_min;
+	g_return_val_if_fail (GDATA_IS_QUERY (self), -1);
+	return self->priv->updated_min;
 }
 
 /**
  * gdata_query_set_updated_min:
  * @self: a #GDataQuery
- * @updated_min: the new minimum update time, or %NULL
+ * @updated_min: the new minimum update time, or <code class="literal">-1</code>
  *
  * Sets the #GDataQuery:updated-min property of the #GDataQuery to the new minimum update time, @updated_min.
  *
- * Set @updated_min to %NULL to unset the property in the query URI.
+ * Set @updated_min to <code class="literal">-1</code> to unset the property in the query URI.
  **/
 void
-gdata_query_set_updated_min (GDataQuery *self, GTimeVal *updated_min)
+gdata_query_set_updated_min (GDataQuery *self, gint64 updated_min)
 {
 	g_return_if_fail (GDATA_IS_QUERY (self));
+	g_return_if_fail (updated_min >= -1);
 
-	if (updated_min == NULL) {
-		self->priv->updated_min.tv_sec = 0;
-		self->priv->updated_min.tv_usec = 0;
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_UPDATED_MIN;
-	} else {
-		self->priv->updated_min = *updated_min;
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_UPDATED_MIN;
-	}
-
+	self->priv->updated_min = updated_min;
 	g_object_notify (G_OBJECT (self), "updated-min");
 
 	/* Our current ETag will no longer be relevant */
@@ -785,42 +700,34 @@ gdata_query_set_updated_min (GDataQuery *self, GTimeVal *updated_min)
 /**
  * gdata_query_get_updated_max:
  * @self: a #GDataQuery
- * @updated_max: a #GTimeVal
  *
- * Gets the #GDataQuery:updated-max property and puts it in @updated_max. If the property is unset,
- * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
+ * Gets the #GDataQuery:updated-max property. If the property is unset, <code class="literal">-1</code> will be returned.
+ *
+ * Return value: the updated-max property, or <code class="literal">-1</code>
  **/
-void
-gdata_query_get_updated_max (GDataQuery *self, GTimeVal *updated_max)
+gint64
+gdata_query_get_updated_max (GDataQuery *self)
 {
-	g_return_if_fail (GDATA_IS_QUERY (self));
-	g_return_if_fail (updated_max != NULL);
-	*updated_max = self->priv->updated_max;
+	g_return_val_if_fail (GDATA_IS_QUERY (self), -1);
+	return self->priv->updated_max;
 }
 
 /**
  * gdata_query_set_updated_max:
  * @self: a #GDataQuery
- * @updated_max: the new maximum update time, or %NULL
+ * @updated_max: the new maximum update time, or <code class="literal">-1</code>
  *
  * Sets the #GDataQuery:updated-max property of the #GDataQuery to the new maximum update time, @updated_max.
  *
- * Set @updated_max to %NULL to unset the property in the query URI.
+ * Set @updated_max to <code class="literal">-1</code> to unset the property in the query URI.
  **/
 void
-gdata_query_set_updated_max (GDataQuery *self, GTimeVal *updated_max)
+gdata_query_set_updated_max (GDataQuery *self, gint64 updated_max)
 {
 	g_return_if_fail (GDATA_IS_QUERY (self));
+	g_return_if_fail (updated_max >= -1);
 
-	if (updated_max == NULL) {
-		self->priv->updated_max.tv_sec = 0;
-		self->priv->updated_max.tv_usec = 0;
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_UPDATED_MAX;
-	} else {
-		self->priv->updated_max = *updated_max;
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_UPDATED_MAX;
-	}
-
+	self->priv->updated_max = updated_max;
 	g_object_notify (G_OBJECT (self), "updated-max");
 
 	/* Our current ETag will no longer be relevant */
@@ -830,42 +737,34 @@ gdata_query_set_updated_max (GDataQuery *self, GTimeVal *updated_max)
 /**
  * gdata_query_get_published_min:
  * @self: a #GDataQuery
- * @published_min: a #GTimeVal
  *
- * Gets the #GDataQuery:published-min property and puts it in @published_min. If the property is unset,
- * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
+ * Gets the #GDataQuery:published-min property. If the property is unset, <code class="literal">-1</code> will be returned.
+ *
+ * Return value: the published-min property, or <code class="literal">-1</code>
  **/
-void
-gdata_query_get_published_min (GDataQuery *self, GTimeVal *published_min)
+gint64
+gdata_query_get_published_min (GDataQuery *self)
 {
-	g_return_if_fail (GDATA_IS_QUERY (self));
-	g_return_if_fail (published_min != NULL);
-	*published_min = self->priv->published_min;
+	g_return_val_if_fail (GDATA_IS_QUERY (self), -1);
+	return self->priv->published_min;
 }
 
 /**
  * gdata_query_set_published_min:
  * @self: a #GDataQuery
- * @published_min: the new minimum publish time, or %NULL
+ * @published_min: the new minimum publish time, or <code class="literal">-1</code>
  *
  * Sets the #GDataQuery:published-min property of the #GDataQuery to the new minimum publish time, @published_min.
  *
- * Set @published_min to %NULL to unset the property in the query URI.
+ * Set @published_min to <code class="literal">-1</code> to unset the property in the query URI.
  **/
 void
-gdata_query_set_published_min (GDataQuery *self, GTimeVal *published_min)
+gdata_query_set_published_min (GDataQuery *self, gint64 published_min)
 {
 	g_return_if_fail (GDATA_IS_QUERY (self));
+	g_return_if_fail (published_min >= -1);
 
-	if (published_min == NULL) {
-		self->priv->published_min.tv_sec = 0;
-		self->priv->published_min.tv_usec = 0;
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_PUBLISHED_MIN;
-	} else {
-		self->priv->published_min = *published_min;
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_PUBLISHED_MIN;
-	}
-
+	self->priv->published_min = published_min;
 	g_object_notify (G_OBJECT (self), "published-min");
 
 	/* Our current ETag will no longer be relevant */
@@ -875,42 +774,34 @@ gdata_query_set_published_min (GDataQuery *self, GTimeVal *published_min)
 /**
  * gdata_query_get_published_max:
  * @self: a #GDataQuery
- * @published_max: a #GTimeVal
  *
- * Gets the #GDataQuery:published-max property and puts it in @published_max. If the property is unset,
- * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
+ * Gets the #GDataQuery:published-max property. If the property is unset, <code class="literal">-1</code> will be returned.
+ *
+ * Return value: the published-max property, or <code class="literal">-1</code>
  **/
-void
-gdata_query_get_published_max (GDataQuery *self, GTimeVal *published_max)
+gint64
+gdata_query_get_published_max (GDataQuery *self)
 {
-	g_return_if_fail (GDATA_IS_QUERY (self));
-	g_return_if_fail (published_max != NULL);
-	*published_max = self->priv->published_max;
+	g_return_val_if_fail (GDATA_IS_QUERY (self), -1);
+	return self->priv->published_max;
 }
 
 /**
  * gdata_query_set_published_max:
  * @self: a #GDataQuery
- * @published_max: the new maximum publish time, or %NULL
+ * @published_max: the new maximum publish time, or <code class="literal">-1</code>
  *
  * Sets the #GDataQuery:published-max property of the #GDataQuery to the new maximum publish time, @published_max.
  *
- * Set @published_max to %NULL to unset the property in the query URI.
+ * Set @published_max to <code class="literal">-1</code> to unset the property in the query URI.
  **/
 void
-gdata_query_set_published_max (GDataQuery *self, GTimeVal *published_max)
+gdata_query_set_published_max (GDataQuery *self, gint64 published_max)
 {
 	g_return_if_fail (GDATA_IS_QUERY (self));
+	g_return_if_fail (published_max >= -1);
 
-	if (published_max == NULL) {
-		self->priv->published_max.tv_sec = 0;
-		self->priv->published_max.tv_usec = 0;
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_PUBLISHED_MAX;
-	} else {
-		self->priv->published_max = *published_max;
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_PUBLISHED_MAX;
-	}
-
+	self->priv->published_max = published_max;
 	g_object_notify (G_OBJECT (self), "published-max");
 
 	/* Our current ETag will no longer be relevant */
@@ -923,41 +814,30 @@ gdata_query_set_published_max (GDataQuery *self, GTimeVal *published_max)
  *
  * Gets the #GDataQuery:start-index property.
  *
- * Return value: the start index property, or <code class="literal">-1</code> if it is unset
+ * Return value: the start index property, or <code class="literal">0</code> if it is unset
  **/
-gint
+guint
 gdata_query_get_start_index (GDataQuery *self)
 {
-	g_return_val_if_fail (GDATA_IS_QUERY (self), -1);
+	g_return_val_if_fail (GDATA_IS_QUERY (self), 0);
 	return self->priv->start_index;
 }
 
 /**
  * gdata_query_set_start_index:
  * @self: a #GDataQuery
- * @start_index: the new start index
+ * @start_index: the new start index, or <code class="literal">0</code>
  *
  * Sets the #GDataQuery:start-index property of the #GDataQuery to the new one-based start index, @start_index.
  *
- * Set @start_index to <code class="literal">-1</code> or <code class="literal">0</code> to unset the property in the query URI.
+ * Set @start_index to <code class="literal">0</code> to unset the property in the query URI.
  **/
 void
-gdata_query_set_start_index (GDataQuery *self, gint start_index)
+gdata_query_set_start_index (GDataQuery *self, guint start_index)
 {
 	g_return_if_fail (GDATA_IS_QUERY (self));
-	g_return_if_fail (start_index >= -1);
-
-	/* Normalise it first */
-	if (start_index <= 0)
-		start_index = -1;
 
 	self->priv->start_index = start_index;
-
-	if (start_index == -1)
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_START_INDEX;
-	else
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_START_INDEX;
-
 	g_object_notify (G_OBJECT (self), "start-index");
 
 	/* Our current ETag will no longer be relevant */
@@ -996,12 +876,6 @@ gdata_query_set_is_strict (GDataQuery *self, gboolean is_strict)
 	g_return_if_fail (GDATA_IS_QUERY (self));
 
 	self->priv->is_strict = is_strict;
-
-	if (is_strict == FALSE)
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_IS_STRICT;
-	else
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_IS_STRICT;
-
 	g_object_notify (G_OBJECT (self), "is-strict");
 
 	/* Our current ETag will no longer be relevant */
@@ -1014,86 +888,36 @@ gdata_query_set_is_strict (GDataQuery *self, gboolean is_strict)
  *
  * Gets the #GDataQuery:max-results property.
  *
- * Return value: the maximum results property, or <code class="literal">-1</code> if it is unset
+ * Return value: the maximum results property, or <code class="literal">0</code> if it is unset
  **/
-gint
+guint
 gdata_query_get_max_results (GDataQuery *self)
 {
-	g_return_val_if_fail (GDATA_IS_QUERY (self), -1);
+	g_return_val_if_fail (GDATA_IS_QUERY (self), 0);
 	return self->priv->max_results;
 }
 
 /**
  * gdata_query_set_max_results:
  * @self: a #GDataQuery
- * @max_results: the new maximum results value
+ * @max_results: the new maximum results value, or <code class="literal">0</code>
  *
  * Sets the #GDataQuery:max-results property of the #GDataQuery to the new maximum results value, @max_results.
  *
- * Set @max_results to <code class="literal">-1</code> to unset the property in the query URI.
+ * Set @max_results to <code class="literal">0</code> to unset the property in the query URI.
  **/
 void
-gdata_query_set_max_results (GDataQuery *self, gint max_results)
+gdata_query_set_max_results (GDataQuery *self, guint max_results)
 {
 	g_return_if_fail (GDATA_IS_QUERY (self));
-	g_return_if_fail (max_results >= -1);
 
 	self->priv->max_results = max_results;
-
-	if (max_results == -1)
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_MAX_RESULTS;
-	else
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_MAX_RESULTS;
-
 	g_object_notify (G_OBJECT (self), "max-results");
 
 	/* Our current ETag will no longer be relevant */
 	gdata_query_set_etag (self, NULL);
 }
 
-/**
- * gdata_query_get_entry_id:
- * @self: a #GDataQuery
- *
- * Gets the #GDataQuery:entry-id property.
- *
- * Return value: the entry ID property, or %NULL if it is unset
- **/
-const gchar *
-gdata_query_get_entry_id (GDataQuery *self)
-{
-	g_return_val_if_fail (GDATA_IS_QUERY (self), NULL);
-	return self->priv->entry_id;
-}
-
-/**
- * gdata_query_set_entry_id:
- * @self: a #GDataQuery
- * @entry_id: the new entry ID string
- *
- * Sets the #GDataQuery:entry-id property of the #GDataQuery to the new entry ID string, @entry_id.
- *
- * Set @entry_id to %NULL to unset the property in the query URI.
- **/
-void
-gdata_query_set_entry_id (GDataQuery *self, const gchar *entry_id)
-{
-	g_return_if_fail (GDATA_IS_QUERY (self));
-
-	g_free (self->priv->entry_id);
-	self->priv->entry_id = g_strdup (entry_id);
-
-	if (entry_id == NULL)
-		self->priv->parameter_mask &= ~GDATA_QUERY_PARAM_ENTRY_ID;
-	else
-		self->priv->parameter_mask |= GDATA_QUERY_PARAM_ENTRY_ID;
-
-	g_object_notify (G_OBJECT (self), "entry-id");
-
-	/* Our current ETag will no longer be relevant */
-	gdata_query_set_etag (self, NULL);
-}
-
 /**
  * gdata_query_get_etag:
  * @self: a #GDataQuery
@@ -1114,7 +938,7 @@ gdata_query_get_etag (GDataQuery *self)
 /**
  * gdata_query_set_etag:
  * @self: a #GDataQuery
- * @etag: the new ETag
+ * @etag: (allow-none): the new ETag, or %NULL
  *
  * Sets the #GDataQuery:etag property of the #GDataQuery to the new ETag, @etag.
  *
@@ -1169,10 +993,14 @@ gdata_query_next_page (GDataQuery *self)
 {
 	GDataQueryPrivate *priv = self->priv;
 
+	g_return_if_fail (GDATA_IS_QUERY (self));
+
 	if (priv->next_uri != NULL) {
 		priv->use_next_uri = TRUE;
 		priv->use_previous_uri = FALSE;
 	} else {
+		if (priv->start_index == 0)
+			priv->start_index++;
 		priv->start_index += priv->max_results;
 	}
 
@@ -1196,13 +1024,17 @@ gdata_query_previous_page (GDataQuery *self)
 {
 	GDataQueryPrivate *priv = self->priv;
 
+	g_return_val_if_fail (GDATA_IS_QUERY (self), FALSE);
+
 	if (priv->next_uri != NULL) {
 		priv->use_previous_uri = TRUE;
 		priv->use_next_uri = FALSE;
-	} else if (priv->start_index < priv->max_results) {
+	} else if (priv->start_index <= priv->max_results) {
 		return FALSE;
 	} else {
 		priv->start_index -= priv->max_results;
+		if (priv->start_index == 1)
+			priv->start_index--;
 	}
 
 	/* Our current ETag will no longer be relevant */
