@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * GData Client
- * Copyright (C) Philip Withnall 2009 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2009–2010 <philip@tecnocode.co.uk>
  *
  * GData Client is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,7 +17,7 @@
  * License along with GData Client.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/**
+/*
  * SECTION:gdata-media-group
  * @short_description: Media RSS group element
  * @stability: Unstable
@@ -31,7 +31,7 @@
  * for most of its properties, and it would be unnecessary and confusing to expose #GDataMediaGroup itself.
  *
  * For this reason, properties have not been implemented on #GDataMediaGroup (yet).
- **/
+ */
 
 #include <glib.h>
 #include <libxml/parser.h>
@@ -52,7 +52,7 @@ static void get_xml (GDataParsable *parsable, GString *xml_string);
 static void get_namespaces (GDataParsable *parsable, GHashTable *namespaces);
 
 struct _GDataMediaGroupPrivate {
-	gchar *keywords;
+	gchar **keywords;
 	gchar *player_uri;
 	GHashTable *restricted_countries;
 	GList *thumbnails; /* GDataMediaThumbnail */
@@ -64,7 +64,6 @@ struct _GDataMediaGroupPrivate {
 };
 
 G_DEFINE_TYPE (GDataMediaGroup, gdata_media_group, GDATA_TYPE_PARSABLE)
-#define GDATA_MEDIA_GROUP_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_MEDIA_GROUP, GDataMediaGroupPrivate))
 
 static void
 gdata_media_group_class_init (GDataMediaGroupClass *klass)
@@ -125,7 +124,7 @@ gdata_media_group_finalize (GObject *object)
 {
 	GDataMediaGroupPrivate *priv = GDATA_MEDIA_GROUP (object)->priv;
 
-	g_free (priv->keywords);
+	g_strfreev (priv->keywords);
 	g_free (priv->player_uri);
 	g_hash_table_destroy (priv->restricted_countries);
 	g_free (priv->title);
@@ -138,120 +137,116 @@ gdata_media_group_finalize (GObject *object)
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
+	gboolean success;
 	GDataMediaGroup *self = GDATA_MEDIA_GROUP (parsable);
 
-	if (xmlStrcmp (node->name, (xmlChar*) "title") == 0) {
-		/* media:title */
-		xmlChar *title = xmlNodeListGetString (doc, node->children, TRUE);
-		gdata_media_group_set_title (self, (gchar*) title);
-		xmlFree (title);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "description") == 0) {
-		/* media:description */
-		xmlChar *description = xmlNodeListGetString (doc, node->children, TRUE);
-		gdata_media_group_set_description (self, (gchar*) description);
-		xmlFree (description);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "keywords") == 0) {
-		/* media:keywords */
-		xmlChar *keywords = xmlNodeListGetString (doc, node->children, TRUE);
-		gdata_media_group_set_keywords (self, (gchar*) keywords);
-		xmlFree (keywords);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "category") == 0) {
-		/* media:category */
-		GDataMediaCategory *category = GDATA_MEDIA_CATEGORY (_gdata_parsable_new_from_xml_node (GDATA_TYPE_MEDIA_CATEGORY, doc,
-													node, NULL, error));
-		if (category == NULL)
-			return FALSE;
+	if (gdata_parser_is_namespace (node, "http://search.yahoo.com/mrss/") == TRUE) {
+		if (gdata_parser_string_from_element (node, "title", P_NONE, &(self->priv->title), &success, error) == TRUE ||
+		    gdata_parser_string_from_element (node, "description", P_NONE, &(self->priv->description), &success, error) == TRUE ||
+		    gdata_parser_object_from_element_setter (node, "category", P_REQUIRED, GDATA_TYPE_MEDIA_CATEGORY,
+		                                             gdata_media_group_set_category, self, &success, error) == TRUE ||
+		    gdata_parser_object_from_element_setter (node, "content", P_REQUIRED, GDATA_TYPE_MEDIA_CONTENT,
+		                                             _gdata_media_group_add_content, self, &success, error) == TRUE ||
+		    gdata_parser_object_from_element_setter (node, "thumbnail", P_REQUIRED, GDATA_TYPE_MEDIA_THUMBNAIL,
+		                                             _gdata_media_group_add_thumbnail, self, &success, error) == TRUE ||
+		    gdata_parser_object_from_element (node, "credit", P_REQUIRED | P_NO_DUPES, GDATA_TYPE_MEDIA_CREDIT,
+		                                      &(self->priv->credit), &success, error) == TRUE) {
+			return success;
+		} else if (xmlStrcmp (node->name, (xmlChar*) "keywords") == 0) {
+			/* media:keywords */
+			guint i;
+			xmlChar *text = xmlNodeListGetString (node->doc, node->children, TRUE);
 
-		gdata_media_group_set_category (self, category);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "content") == 0) {
-		/* media:content */
-		GDataMediaContent *content = GDATA_MEDIA_CONTENT (_gdata_parsable_new_from_xml_node (GDATA_TYPE_MEDIA_CONTENT, doc, node, NULL, error));
-		if (content == NULL)
-			return FALSE;
+			g_strfreev (self->priv->keywords);
+			if (text == NULL) {
+				self->priv->keywords = NULL;
+				return TRUE;
+			}
 
-		_gdata_media_group_add_content (self, content);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "credit") == 0) {
-		/* media:credit */
-		GDataMediaCredit *credit = GDATA_MEDIA_CREDIT (_gdata_parsable_new_from_xml_node (GDATA_TYPE_MEDIA_CREDIT, doc, node, NULL, error));
-		if (credit == NULL)
-			return FALSE;
+			self->priv->keywords = g_strsplit ((gchar*) text, ",", -1);
 
-		if (self->priv->credit != NULL) {
-			g_object_unref (credit);
-			return gdata_parser_error_duplicate_element (node, error);
-		}
+			for (i = 0; self->priv->keywords[i] != NULL; i++) {
+				gchar *comma, *start = self->priv->keywords[i];
+				gchar *end = start + strlen (start);
 
-		_gdata_media_group_set_credit (self, credit);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "player") == 0) {
-		/* media:player */
-		xmlChar *player_uri = xmlGetProp (node, (xmlChar*) "url");
+				/* Strip any whitespace from the ends of the keyword */
+				g_strstrip (start);
 
-		g_free (self->priv->player_uri);
-		self->priv->player_uri = g_strdup ((gchar*) player_uri);
+				/* Unescape any %2Cs in the keyword to commas in-place */
+				while ((comma = g_strstr_len (start, -1, "%2C")) != NULL) {
+					/* Unescape the comma */
+					*comma = ',';
 
-		xmlFree (player_uri);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "rating") == 0) {
-		/* media:rating */
-		xmlChar *countries;
-		gchar **country_list, **country;
+					/* Move forwards, skipping the comma */
+					comma++;
+					end -= 2;
 
-		countries = xmlGetProp (node, (xmlChar*) "country");
-		country_list = g_strsplit ((const gchar*) countries, ",", -1);
-		xmlFree (countries);
+					/* Shift the remainder of the string downwards */
+					g_memmove (comma, comma + 2, end - comma);
+					*end = '\0';
+				}
+			}
+		} else if (xmlStrcmp (node->name, (xmlChar*) "player") == 0) {
+			/* media:player */
+			xmlChar *player_uri = xmlGetProp (node, (xmlChar*) "url");
+			g_free (self->priv->player_uri);
+			self->priv->player_uri = (gchar*) player_uri;
+		} else if (xmlStrcmp (node->name, (xmlChar*) "rating") == 0) {
+			/* media:rating */
+			xmlChar *countries;
+			gchar **country_list, **country;
 
-		/* Add all the listed countries to the restricted countries table */
-		for (country = country_list; *country != NULL; country++)
-			g_hash_table_insert (self->priv->restricted_countries, *country, GUINT_TO_POINTER (TRUE));
-		g_free (country_list);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "restriction") == 0) {
-		/* media:restriction */
-		xmlChar *type, *countries, *relationship;
-		gchar **country_list, **country;
-		gboolean relationship_bool;
+			countries = xmlGetProp (node, (xmlChar*) "country");
+			country_list = g_strsplit ((const gchar*) countries, ",", -1);
+			xmlFree (countries);
 
-		/* Check the type property is "country" */
-		type = xmlGetProp (node, (xmlChar*) "type");
-		if (xmlStrcmp (type, (xmlChar*) "country") != 0) {
-			gdata_parser_error_unknown_property_value (node, "type", (gchar*) type, error);
+			/* Add all the listed countries to the restricted countries table */
+			for (country = country_list; *country != NULL; country++)
+				g_hash_table_insert (self->priv->restricted_countries, *country, GUINT_TO_POINTER (TRUE));
+			g_free (country_list);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "restriction") == 0) {
+			/* media:restriction */
+			xmlChar *type, *countries, *relationship;
+			gchar **country_list, **country;
+			gboolean relationship_bool;
+
+			/* Check the type property is "country" */
+			type = xmlGetProp (node, (xmlChar*) "type");
+			if (xmlStrcmp (type, (xmlChar*) "country") != 0) {
+				gdata_parser_error_unknown_property_value (node, "type", (gchar*) type, error);
+				xmlFree (type);
+				return FALSE;
+			}
 			xmlFree (type);
-			return FALSE;
-		}
-		xmlFree (type);
 
-		relationship = xmlGetProp (node, (xmlChar*) "relationship");
-		if (xmlStrcmp (relationship, (xmlChar*) "allow") == 0)
-			relationship_bool = FALSE; /* it's *not* a restricted country */
-		else if (xmlStrcmp (relationship, (xmlChar*) "deny") == 0)
-			relationship_bool = TRUE; /* it *is* a restricted country */
-		else {
-			gdata_parser_error_unknown_property_value (node, "relationship", (gchar*) relationship, error);
+			relationship = xmlGetProp (node, (xmlChar*) "relationship");
+			if (xmlStrcmp (relationship, (xmlChar*) "allow") == 0) {
+				relationship_bool = FALSE; /* it's *not* a restricted country */
+			} else if (xmlStrcmp (relationship, (xmlChar*) "deny") == 0) {
+				relationship_bool = TRUE; /* it *is* a restricted country */
+			} else {
+				gdata_parser_error_unknown_property_value (node, "relationship", (gchar*) relationship, error);
+				xmlFree (relationship);
+				return FALSE;
+			}
 			xmlFree (relationship);
-			return FALSE;
+
+			countries = xmlNodeListGetString (doc, node->children, TRUE);
+			country_list = g_strsplit ((const gchar*) countries, " ", -1);
+			xmlFree (countries);
+
+			/* Add "all" to the table, since it's an exception table */
+			g_hash_table_insert (self->priv->restricted_countries, g_strdup ("all"), GUINT_TO_POINTER (!relationship_bool));
+
+			/* Add all the listed countries to the restricted countries table */
+			for (country = country_list; *country != NULL; country++)
+				g_hash_table_insert (self->priv->restricted_countries, *country, GUINT_TO_POINTER (relationship_bool));
+			g_free (country_list);
+		} else {
+			return GDATA_PARSABLE_CLASS (gdata_media_group_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 		}
-		xmlFree (relationship);
-
-		countries = xmlNodeListGetString (doc, node->children, TRUE);
-		country_list = g_strsplit ((const gchar*) countries, " ", -1);
-		xmlFree (countries);
-
-		/* Add "all" to the table, since it's an exception table */
-		g_hash_table_insert (self->priv->restricted_countries, g_strdup ("all"), GUINT_TO_POINTER (!relationship_bool));
-
-		/* Add all the listed countries to the restricted countries table */
-		for (country = country_list; *country != NULL; country++)
-			g_hash_table_insert (self->priv->restricted_countries, *country, GUINT_TO_POINTER (relationship_bool));
-		g_free (country_list);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "thumbnail") == 0) {
-		/* media:thumbnail */
-		GDataMediaThumbnail *thumb = GDATA_MEDIA_THUMBNAIL (_gdata_parsable_new_from_xml_node (GDATA_TYPE_MEDIA_THUMBNAIL, doc,
-												       node, NULL, error));
-		if (thumb == NULL)
-			return FALSE;
-
-		self->priv->thumbnails = g_list_prepend (self->priv->thumbnails, thumb);
-	} else if (GDATA_PARSABLE_CLASS (gdata_media_group_parent_class)->parse_xml (parsable, doc, node, user_data, error) == FALSE) {
-		/* Error! */
-		return FALSE;
+	} else {
+		return GDATA_PARSABLE_CLASS (gdata_media_group_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
 
 	return TRUE;
@@ -272,8 +267,39 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 	if (priv->description != NULL)
 		gdata_parser_string_append_escaped (xml_string, "<media:description type='plain'>", priv->description, "</media:description>");
 
-	if (priv->keywords != NULL)
-		gdata_parser_string_append_escaped (xml_string, "<media:keywords>", priv->keywords, "</media:keywords>");
+	if (priv->keywords != NULL) {
+		guint i;
+
+		g_string_append (xml_string, "<media:keywords>");
+
+		/* Add each keyword to the text content, comma-separated from the previous one */
+		for (i = 0; priv->keywords[i] != NULL; i++) {
+			const gchar *comma, *start = priv->keywords[i];
+
+			/* Delimit the previous keyword */
+			if (i != 0)
+				g_string_append_c (xml_string, ',');
+
+			/* Escape any commas in the keyword to %2C */
+			while ((comma = g_utf8_strchr (start, -1, ',')) != NULL) {
+				/* Copy the span */
+				gchar *span = g_strndup (start, comma - start);
+				g_string_append (xml_string, span);
+				g_free (span);
+
+				/* Add an escaped comma */
+				g_string_append (xml_string, "%2C");
+
+				/* Move forwards, skipping the comma */
+				start = comma + 1;
+			}
+
+			/* Append the rest of the string (the entire string if there were no commas) */
+			g_string_append (xml_string, start);
+		}
+
+		g_string_append (xml_string, "</media:keywords>");
+	}
 }
 
 static void
@@ -302,7 +328,7 @@ gdata_media_group_get_title (GDataMediaGroup *self)
 /**
  * gdata_media_group_set_title:
  * @self: a #GDataMediaGroup
- * @title: the group's new title, or %NULL
+ * @title: (allow-none): the group's new title, or %NULL
  *
  * Sets the #GDataMediaGroup:title property to @title.
  *
@@ -338,7 +364,7 @@ gdata_media_group_get_description (GDataMediaGroup *self)
 /**
  * gdata_media_group_set_description:
  * @self: a #GDataMediaGroup
- * @description: the group's new description, or %NULL
+ * @description: (allow-none): the group's new description, or %NULL
  *
  * Sets the #GDataMediaGroup:description property to @description.
  *
@@ -360,21 +386,21 @@ gdata_media_group_set_description (GDataMediaGroup *self, const gchar *descripti
  *
  * Gets the #GDataMediaGroup:keywords property.
  *
- * Return value: the group's keywords, or %NULL
+ * Return value: (array zero-terminated=1): a %NULL-terminated array of the group's keywords, or %NULL
  *
  * Since: 0.4.0
  **/
-const gchar *
+const gchar * const *
 gdata_media_group_get_keywords (GDataMediaGroup *self)
 {
 	g_return_val_if_fail (GDATA_IS_MEDIA_GROUP (self), NULL);
-	return self->priv->keywords;
+	return (const gchar * const *) self->priv->keywords;
 }
 
 /**
  * gdata_media_group_set_keywords:
  * @self: a #GDataMediaGroup
- * @keywords: the group's new keywords, or %NULL
+ * @keywords: (array zero-terminated=1) (allow-none): a %NULL-terminated array of the group's new keywords, or %NULL
  *
  * Sets the #GDataMediaGroup:keywords property to @keywords.
  *
@@ -383,11 +409,11 @@ gdata_media_group_get_keywords (GDataMediaGroup *self)
  * Since: 0.4.0
  **/
 void
-gdata_media_group_set_keywords (GDataMediaGroup *self, const gchar *keywords)
+gdata_media_group_set_keywords (GDataMediaGroup *self, const gchar * const *keywords)
 {
 	g_return_if_fail (GDATA_IS_MEDIA_GROUP (self));
-	g_free (self->priv->keywords);
-	self->priv->keywords = g_strdup (keywords);
+	g_strfreev (self->priv->keywords);
+	self->priv->keywords = g_strdupv ((gchar**) keywords);
 }
 
 /**
@@ -408,7 +434,7 @@ gdata_media_group_get_category (GDataMediaGroup *self)
 /**
  * gdata_media_group_set_category:
  * @self: a #GDataMediaGroup
- * @category: a new #GDataMediaCategory, or %NULL
+ * @category: (allow-none): a new #GDataMediaCategory, or %NULL
  *
  * Sets the #GDataMediaGroup:category property to @category, and increments its reference count.
  **/
@@ -437,7 +463,7 @@ content_compare_cb (const GDataMediaContent *content, const gchar *type)
  * Looks up a #GDataMediaContent from the group with the given MIME type. The group's list of contents is
  * a list of URIs to various formats of the group content itself, such as the SWF URI or RTSP stream for a video.
  *
- * Return value: a #GDataMediaContent matching @type, or %NULL
+ * Return value: (transfer none): a #GDataMediaContent matching @type, or %NULL
  **/
 GDataMediaContent *
 gdata_media_group_look_up_content (GDataMediaGroup *self, const gchar *type)
@@ -460,7 +486,7 @@ gdata_media_group_look_up_content (GDataMediaGroup *self, const gchar *type)
  *
  * Returns a list of #GDataMediaContent<!-- -->s, giving the content enclosed by the group.
  *
- * Return value: a #GList of #GDataMediaContent<!-- -->s,  or %NULL
+ * Return value: (element-type GData.MediaContent) (transfer none): a #GList of #GDataMediaContent<!-- -->s,  or %NULL
  **/
 GList *
 gdata_media_group_get_contents (GDataMediaGroup *self)
@@ -472,7 +498,9 @@ gdata_media_group_get_contents (GDataMediaGroup *self)
 void
 _gdata_media_group_add_content (GDataMediaGroup *self, GDataMediaContent *content)
 {
-	self->priv->contents = g_list_prepend (self->priv->contents, content);
+	g_return_if_fail (GDATA_IS_MEDIA_GROUP (self));
+	g_return_if_fail (GDATA_IS_MEDIA_CONTENT (content));
+	self->priv->contents = g_list_prepend (self->priv->contents, g_object_ref (content));
 }
 
 /**
@@ -493,7 +521,12 @@ gdata_media_group_get_credit (GDataMediaGroup *self)
 void
 _gdata_media_group_set_credit (GDataMediaGroup *self, GDataMediaCredit *credit)
 {
-	self->priv->credit = credit;
+	g_return_if_fail (GDATA_IS_MEDIA_GROUP (self));
+	g_return_if_fail (credit == NULL ||GDATA_IS_MEDIA_CREDIT (credit));
+
+	if (self->priv->credit != NULL)
+		g_object_unref (self->priv->credit);
+	self->priv->credit = g_object_ref (credit);
 }
 
 /**
@@ -539,7 +572,7 @@ gdata_media_group_is_restricted_in_country (GDataMediaGroup *self, const gchar *
  *
  * Gets a list of the thumbnails available for the group.
  *
- * Return value: a #GList of #GDataMediaThumbnail<!-- -->s, or %NULL
+ * Return value: (element-type GData.MediaThumbnail) (transfer none): a #GList of #GDataMediaThumbnail<!-- -->s, or %NULL
  **/
 GList *
 gdata_media_group_get_thumbnails (GDataMediaGroup *self)
@@ -547,3 +580,11 @@ gdata_media_group_get_thumbnails (GDataMediaGroup *self)
 	g_return_val_if_fail (GDATA_IS_MEDIA_GROUP (self), NULL);
 	return self->priv->thumbnails;
 }
+
+void
+_gdata_media_group_add_thumbnail (GDataMediaGroup *self, GDataMediaThumbnail *thumbnail)
+{
+	g_return_if_fail (GDATA_IS_MEDIA_GROUP (self));
+	g_return_if_fail (GDATA_IS_MEDIA_THUMBNAIL (thumbnail));
+	self->priv->thumbnails = g_list_prepend (self->priv->thumbnails, g_object_ref (thumbnail));
+}
