@@ -30,6 +30,55 @@
  * For more details of PicasaWeb's GData API, see the <ulink type="http" url="http://code.google.com/apis/picasaweb/developers_guide_protocol.html">
  * online documentation</ulink>.
  *
+ * <example>
+ * 	<title>Authenticating and Creating a New Album</title>
+ * 	<programlisting>
+ *	GDataPicasaWebService *service;
+ *	GDataPicasaWebAlbum *album, *inserted_album;
+ *
+ *	/<!-- -->* Create a service object and authenticate with the PicasaWeb server *<!-- -->/
+ *	service = gdata_picasaweb_service_new ("companyName-applicationName-versionID");
+ *	gdata_service_authenticate (GDATA_SERVICE (service), username, password, NULL, NULL);
+ *
+ *	/<!-- -->* Create a GDataPicasaWebAlbum entry for the new album, setting some information about it *<!-- -->/
+ *	album = gdata_picasaweb_album_new (NULL);
+ *	gdata_entry_set_title (GDATA_ENTRY (album), "Photos from the Rhine");
+ *	gdata_entry_set_summary (GDATA_ENTRY (album), "An album of our adventures on the great river.");
+ *	gdata_picasaweb_album_set_location (album, "The Rhine, Germany");
+ *
+ *	/<!-- -->* Insert the new album on the server. Note that this is a blocking operation. *<!-- -->/
+ *	inserted_album = gdata_picasaweb_service_insert_album (service, album, NULL, NULL);
+ *
+ *	g_object_unref (album);
+ *	g_object_unref (inserted_album);
+ *	g_object_unref (service);
+ *	</programlisting>
+ * </example>
+ *
+ * <example>
+ * 	<title>Uploading a Photo or Video</title>
+ * 	<programlisting>
+ *	GDataPicasaWebFile *file_entry, *uploaded_file_entry;
+ *	GFile *file_data;
+ *
+ *	/<!-- -->* Specify the GFile image on disk to upload *<!-- -->/
+ *	file_data = g_file_new_for_path (path);
+ *
+ *	/<!-- -->* Create a GDataPicasaWebFile entry for the image, setting a title and caption/summary *<!-- -->/
+ *	file_entry = gdata_picasaweb_file_new (NULL);
+ *	gdata_entry_set_title (GDATA_ENTRY (file_entry), "Black Cat");
+ *	gdata_entry_set_summary (GDATA_ENTRY (file_entry), "Photo of the world's most beautiful cat.");
+ *
+ *	/<!-- -->* Upload the file to the server. Note that this is a blocking operation. *<!-- -->/
+ *	uploaded_file_entry = gdata_picasaweb_service_upload_file (service, album, file_entry, file_data, NULL, NULL);
+ *
+ *	g_object_unref (file_entry);
+ *	g_object_unref (uploaded_file_entry);
+ *	g_object_unref (file_data);
+ * 	</programlisting>
+ * </example>
+ *
+ *
  * Since: 0.4.0
  **/
 
@@ -61,10 +110,29 @@ gdata_picasaweb_service_init (GDataPicasaWebService *self)
 	/* Nothing to see here */
 }
 
+/**
+ * gdata_picasaweb_service_new:
+ * @client_id: your application's client ID
+ *
+ * Creates a new #GDataPicasaWebService. The @client_id must be unique for your application, and as registered with Google.
+ * The <ulink type="http" url="http://code.google.com/apis/accounts/docs/AuthForInstalledApps.html#Request">recommended
+ * form</ulink> is "companyName-applicationName-versionID".
+ *
+ * Return value: a new #GDataPicasaWebService, or %NULL
+ *
+ * Since: 0.4.0
+ **/
+GDataPicasaWebService *
+gdata_picasaweb_service_new (const gchar *client_id)
+{
+	g_return_val_if_fail (client_id != NULL, NULL);
+	return g_object_new (GDATA_TYPE_PICASAWEB_SERVICE, "client-id", client_id, NULL);
+}
+
 /*
  * create_uri:
  * @self: a #GDataPicasaWebService
- * @username: the username to use, or %NULL to use the currently logged in user
+ * @username: (allow-none): the username to use, or %NULL to use the currently logged in user
  * @type: the type of object to access: "entry" for a user, or "feed" for an album
  *
  * Builds a URI to use when querying for albums or a user.
@@ -85,39 +153,19 @@ create_uri (GDataPicasaWebService *self, const gchar *username, const gchar *typ
 		username = "default";
 	}
 
-	return g_strdup_printf ("http://picasaweb.google.com/data/%s/api/user/%s", type, username);
+	return _gdata_service_build_uri (TRUE, "http://picasaweb.google.com/data/%s/api/user/%s", type, username);
 }
 
 /**
- * gdata_picasaweb_service_new:
- * @client_id: your application's client ID
- *
- * Creates a new #GDataPicasaWebService. The @client_id must be unique for your application, and as registered with Google.
- *
- * Return value: a new #GDataPicasaWebService, or %NULL
- *
- * Since: 0.4.0
- **/
-GDataPicasaWebService *
-gdata_picasaweb_service_new (const gchar *client_id)
-{
-	g_return_val_if_fail (client_id != NULL, NULL);
-
-	return g_object_new (GDATA_TYPE_PICASAWEB_SERVICE,
-			     "client-id", client_id,
-			     NULL);
-}
-
-/**
- * gdata_picasaweb_service_get_user
+ * gdata_picasaweb_service_get_user:
  * @self: a #GDataPicasaWebService
- * @username: the username of the user whose information you wish to retrieve, or %NULL for the currently authenticated user.
+ * @username: (allow-none): the username of the user whose information you wish to retrieve, or %NULL for the currently authenticated user.
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
  * Queries the service to return the user specified by @username.
  *
- * Return value: a #GDataPicasaWebUser; unref with g_object_unref()
+ * Return value: (transfer full): a #GDataPicasaWebUser; unref with g_object_unref()
  *
  * Since: 0.6.0
  **/
@@ -129,14 +177,19 @@ gdata_picasaweb_service_get_user (GDataPicasaWebService *self, const gchar *user
 	SoupMessage *message;
 
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_SERVICE (self), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	uri = create_uri (self, username, "entry");
 	if (uri == NULL) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must specify a username or be authenticated to query a user."));
+		                     _("You must specify a username or be authenticated to query a user."));
 		return NULL;
 	}
-	message = _gdata_service_query (GDATA_SERVICE (self), uri, NULL, cancellable, NULL, NULL, error);
+
+	message = _gdata_service_query (GDATA_SERVICE (self), uri, NULL, cancellable, error);
+	g_free (uri);
+
 	if (message == NULL)
 		return NULL;
 
@@ -150,11 +203,11 @@ gdata_picasaweb_service_get_user (GDataPicasaWebService *self, const gchar *user
 /**
  * gdata_picasaweb_service_query_all_albums:
  * @self: a #GDataPicasaWebService
- * @query: a #GDataQuery with the query parameters, or %NULL
- * @username: the username of the user whose albums you wish to retrieve, or %NULL
+ * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
+ * @username: (allow-none): the username of the user whose albums you wish to retrieve, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
- * @progress_user_data: data to pass to the @progress_callback function
+ * @progress_callback: (scope call): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @error: a #GError, or %NULL
  *
  * Queries the service to return a list of all albums belonging to the specified @username which match the given
@@ -165,52 +218,54 @@ gdata_picasaweb_service_get_user (GDataPicasaWebService *self, const gchar *user
  *
  * For more details, see gdata_service_query().
  *
- * Return value: a #GDataFeed of query results; unref with g_object_unref()
+ * Return value: (transfer full): a #GDataFeed of query results; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
 GDataFeed *
 gdata_picasaweb_service_query_all_albums (GDataPicasaWebService *self, GDataQuery *query, const gchar *username, GCancellable *cancellable,
-					  GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
+                                          GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
 	gchar *uri;
 	GDataFeed *album_feed;
 
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_SERVICE (self), NULL);
 	g_return_val_if_fail (query == NULL || GDATA_IS_QUERY (query), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	if (query != NULL && gdata_query_get_q (query) != NULL) {
 		/* Bug #593336 — Query parameter "q=..." isn't valid for album kinds */
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER,
-				     _("Query parameter not allowed for albums."));
+		                     _("Query parameter not allowed for albums."));
 		return NULL;
 	}
 
 	uri = create_uri (self, username, "feed");
 	if (uri == NULL) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must specify a username or be authenticated to query all albums."));
+		                     _("You must specify a username or be authenticated to query all albums."));
 		return NULL;
 	}
 
 	/* Execute the query */
 	album_feed = gdata_service_query (GDATA_SERVICE (self), uri, query, GDATA_TYPE_PICASAWEB_ALBUM,
-					  cancellable, progress_callback, progress_user_data, error);
+	                                  cancellable, progress_callback, progress_user_data, error);
 	g_free (uri);
 
 	return album_feed;
 }
 
 /**
- * gdata_picasaweb_service_query_all_albums_async:
+ * gdata_picasaweb_service_query_all_albums_async: (skip)
  * @self: a #GDataPicasaWebService
- * @query: a #GDataQuery with the query parameters, or %NULL
- * @username: the username of the user whose albums you wish to retrieve, or %NULL
+ * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
+ * @username: (allow-none): the username of the user whose albums you wish to retrieve, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
  * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
- * @progress_user_data: data to pass to the @progress_callback function
+ * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @callback: a #GAsyncReadyCallback to call when authentication is finished
- * @user_data: data to pass to the @callback function
+ * @user_data: (closure): data to pass to the @callback function
  *
  * Queries the service to return a list of all albums belonging to the specified @username which match the given
  * @query. @self, @query and @username are all reffed/copied when this function is called, so can safely be unreffed/freed after
@@ -223,45 +278,46 @@ gdata_picasaweb_service_query_all_albums (GDataPicasaWebService *self, GDataQuer
  **/
 void
 gdata_picasaweb_service_query_all_albums_async (GDataPicasaWebService *self, GDataQuery *query, const gchar *username,
-						GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
-						GAsyncReadyCallback callback, gpointer user_data)
+                                                GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
+                                                GAsyncReadyCallback callback, gpointer user_data)
 {
 	gchar *uri;
 
 	g_return_if_fail (GDATA_IS_PICASAWEB_SERVICE (self));
 	g_return_if_fail (query == NULL || GDATA_IS_QUERY (query));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 	g_return_if_fail (callback != NULL);
 
 	if (query != NULL && gdata_query_get_q (query) != NULL) {
 		/* Bug #593336 — Query parameter "q=..." isn't valid for album kinds */
 		g_simple_async_report_error_in_idle (G_OBJECT (self), callback, user_data,
-						     GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER,
-						     _("Query parameter not allowed for albums."));
+		                                     GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER,
+		                                     _("Query parameter not allowed for albums."));
 		return;
 	}
 
 	uri = create_uri (self, username, "feed");
 	if (uri == NULL) {
 		g_simple_async_report_error_in_idle (G_OBJECT (self), callback, user_data,
-						     GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-						     _("You must specify a username or be authenticated to query all albums."));
+		                                     GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
+		                                     _("You must specify a username or be authenticated to query all albums."));
 		return;
 	}
 
 	/* Schedule the async query */
 	gdata_service_query_async (GDATA_SERVICE (self), uri, query, GDATA_TYPE_PICASAWEB_ALBUM, cancellable, progress_callback, progress_user_data,
-				   callback, user_data);
+	                           callback, user_data);
 	g_free (uri);
 }
 
 /**
  * gdata_picasaweb_service_query_files:
  * @self: a #GDataPicasaWebService
- * @album: a #GDataPicasaWebAlbum from which to retrieve the files, or %NULL
- * @query: a #GDataQuery with the query parameters, or %NULL
+ * @album: (allow-none): a #GDataPicasaWebAlbum from which to retrieve the files, or %NULL
+ * @query: (allow-none): a #GDataQuery with the query parameters, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
- * @progress_user_data: data to pass to the @progress_callback function
+ * @progress_callback: (scope call): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @error: a #GError, or %NULL
  *
  * Queries the specified @album for a list of the files which match the given @query. If @album is %NULL and a user is
@@ -269,26 +325,32 @@ gdata_picasaweb_service_query_all_albums_async (GDataPicasaWebService *self, GDa
  *
  * For more details, see gdata_service_query().
  *
- * Return value: a #GDataFeed of query results; unref with g_object_unref()
+ * Return value: (transfer full): a #GDataFeed of query results; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
 GDataFeed *
 gdata_picasaweb_service_query_files (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataQuery *query, GCancellable *cancellable,
-				     GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
+                                     GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
 	/* TODO: Async variant */
 	const gchar *uri;
 
+	g_return_val_if_fail (GDATA_IS_PICASAWEB_SERVICE (self), NULL);
+	g_return_val_if_fail (album == NULL || GDATA_IS_PICASAWEB_ALBUM (album), NULL);
+	g_return_val_if_fail (query == NULL || GDATA_IS_QUERY (query), NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
 	if (album != NULL) {
-		GDataLink *link = gdata_entry_look_up_link (GDATA_ENTRY (album), "http://schemas.google.com/g/2005#feed");
-		if (link == NULL) {
+		GDataLink *_link = gdata_entry_look_up_link (GDATA_ENTRY (album), "http://schemas.google.com/g/2005#feed");
+		if (_link == NULL) {
 			/* Error */
 			g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-					     _("The album did not have a feed link."));
+			                     _("The album did not have a feed link."));
 			return NULL;
 		}
-		uri = gdata_link_get_uri (link);
+		uri = gdata_link_get_uri (_link);
 	} else {
 		/* Default URI */
 		uri = "http://picasaweb.google.com/data/feed/api/user/default/albumid/default";
@@ -296,23 +358,17 @@ gdata_picasaweb_service_query_files (GDataPicasaWebService *self, GDataPicasaWeb
 
 	/* Execute the query */
 	return gdata_service_query (GDATA_SERVICE (self), uri, GDATA_QUERY (query), GDATA_TYPE_PICASAWEB_FILE, cancellable,
-				    progress_callback, progress_user_data, error);
+	                            progress_callback, progress_user_data, error);
 }
 
 static GOutputStream *
 get_file_output_stream (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataPicasaWebFile *file_entry, GFile *file_data, GError **error)
 {
-	GDataCategory *category;
 	GFileInfo *file_info = NULL;
 	const gchar *slug = NULL, *content_type = NULL, *user_id = NULL, *album_id = NULL;
 	GOutputStream *output_stream;
 	gchar *upload_uri;
 
-	/* Add the "photo" kind if the entry is missing it. If it already has the kind category, no duplicate is added. */
-	category = gdata_category_new ("http://schemas.google.com/photos/2007#photo", "http://schemas.google.com/g/2005#kind", NULL);
-	gdata_entry_add_category (GDATA_ENTRY (file_entry), category);
-	g_object_unref (category);
-
 	/* PicasaWeb allows you to post to a default Dropbox */
 	album_id = (album != NULL) ? gdata_entry_get_id (GDATA_ENTRY (album)) : "default";
 	user_id = gdata_service_get_username (GDATA_SERVICE (self));
@@ -325,7 +381,7 @@ get_file_output_stream (GDataPicasaWebService *self, GDataPicasaWebAlbum *album,
 	content_type = g_file_info_get_content_type (file_info);
 
 	/* Build the upload URI and upload stream */
-	upload_uri = g_strdup_printf ("http://picasaweb.google.com/data/feed/api/user/%s/albumid/%s", user_id, album_id);
+	upload_uri = _gdata_service_build_uri (TRUE, "http://picasaweb.google.com/data/feed/api/user/%s/albumid/%s", user_id, album_id);
 	output_stream = gdata_upload_stream_new (GDATA_SERVICE (self), SOUP_METHOD_POST, upload_uri, GDATA_ENTRY (file_entry), slug, content_type);
 	g_free (upload_uri);
 	g_object_unref (file_info);
@@ -353,7 +409,7 @@ parse_spliced_stream (GOutputStream *output_stream, GError **error)
 /**
  * gdata_picasaweb_service_upload_file:
  * @self: a #GDataPicasaWebService
- * @album: a #GDataPicasaWebAlbum into which to insert the file, or %NULL
+ * @album: (allow-none): a #GDataPicasaWebAlbum into which to insert the file, or %NULL
  * @file_entry: a #GDataPicasaWebFile to insert
  * @file_data: the actual file to upload
  * @cancellable: optional #GCancellable object, or %NULL
@@ -368,13 +424,13 @@ parse_spliced_stream (GOutputStream *output_stream, GError **error)
  * If there is a problem reading @file_data, an error from g_output_stream_splice() or g_file_query_info() will be returned. Other errors from
  * #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
  *
- * Return value: the inserted #GDataPicasaWebFile with updated properties from @file_entry; unref with g_object_unref()
+ * Return value: (transfer full): the inserted #GDataPicasaWebFile with updated properties from @file_entry; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
 GDataPicasaWebFile *
 gdata_picasaweb_service_upload_file (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataPicasaWebFile *file_entry, GFile *file_data,
-				     GCancellable *cancellable, GError **error)
+                                     GCancellable *cancellable, GError **error)
 {
 	GOutputStream *output_stream;
 	GInputStream *input_stream;
@@ -385,18 +441,18 @@ gdata_picasaweb_service_upload_file (GDataPicasaWebService *self, GDataPicasaWeb
 	g_return_val_if_fail (album == NULL || GDATA_IS_PICASAWEB_ALBUM (album), NULL);
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_FILE (file_entry), NULL);
 	g_return_val_if_fail (G_IS_FILE (file_data), NULL);
-	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	if (gdata_entry_is_inserted (GDATA_ENTRY (file_entry)) == TRUE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED,
-				     _("The entry has already been inserted."));
+		                     _("The entry has already been inserted."));
 		return NULL;
 	}
 
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must be authenticated to upload a file."));
+		                     _("You must be authenticated to upload a file."));
 		return NULL;
 	}
 
@@ -412,7 +468,7 @@ gdata_picasaweb_service_upload_file (GDataPicasaWebService *self, GDataPicasaWeb
 	}
 
 	g_output_stream_splice (output_stream, input_stream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-				cancellable, &child_error);
+	                        cancellable, &child_error);
 
 	g_object_unref (input_stream);
 	if (child_error != NULL) {
@@ -447,7 +503,7 @@ gdata_picasaweb_service_upload_file (GDataPicasaWebService *self, GDataPicasaWeb
  * no user is authenticated with the service when trying to upload it,
  * %GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED will be set.
  *
- * Return value: the inserted #GDataPicasaWebFile; unref with
+ * Return value: (transfer full): the inserted #GDataPicasaWebFile; unref with
  * g_object_unref()
  *
  * Since: 0.6.0
@@ -456,9 +512,9 @@ GDataPicasaWebFile *
 gdata_picasaweb_service_upload_file_finish (GDataPicasaWebService *self, GAsyncResult *result, GError **error)
 {
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_SERVICE (self), NULL);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-	/* propagate any potential errors we might have encountered in g_output_stream_splice() or gdata_picasaweb_service_upload_file_async() */
 	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
 		return NULL;
 
@@ -495,10 +551,10 @@ upload_file_async_cb (GOutputStream *output_stream, GAsyncResult *result, Upload
 
 	if (error == NULL && file != NULL)
 		async_result = g_simple_async_result_new (G_OBJECT (data->service), (GAsyncReadyCallback) data->callback,
-							  data->user_data, gdata_picasaweb_service_upload_file_async);
+		                                          data->user_data, gdata_picasaweb_service_upload_file_async);
 	else
 		async_result = g_simple_async_result_new_from_error (G_OBJECT (data->service), (GAsyncReadyCallback) data->callback,
-								     data->user_data, error);
+		                                                     data->user_data, error);
 
 	g_simple_async_result_set_op_res_gpointer (async_result, file, NULL);
 
@@ -510,12 +566,12 @@ upload_file_async_cb (GOutputStream *output_stream, GAsyncResult *result, Upload
 /**
  * gdata_picasaweb_service_upload_file_async:
  * @self: a #GDataPicasaWebService
- * @album: a #GDataPicasaWebAlbum into which to insert the file, or %NULL
+ * @album: (allow-none): a #GDataPicasaWebAlbum into which to insert the file, or %NULL
  * @file_entry: a #GDataPicasaWebFile to insert
  * @file_data: the actual file to upload
  * @cancellable: optional #GCancellable object, or %NULL
  * @callback: a #GAsyncReadyCallback to call when authentication is finished
- * @user_data: data to pass to the @callback function
+ * @user_data: (closure): data to pass to the @callback function
  *
  * Uploads a file (photo or video) to the given PicasaWeb @album
  * asynchronously, using the @actual_file from disk and the metadata
@@ -531,7 +587,7 @@ upload_file_async_cb (GOutputStream *output_stream, GAsyncResult *result, Upload
  **/
 void
 gdata_picasaweb_service_upload_file_async (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GDataPicasaWebFile *file_entry,
-					   GFile *file_data, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
+                                           GFile *file_data, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
 {
 	GOutputStream *output_stream;
 	GInputStream *input_stream;
@@ -547,13 +603,13 @@ gdata_picasaweb_service_upload_file_async (GDataPicasaWebService *self, GDataPic
 
 	if (gdata_entry_is_inserted (GDATA_ENTRY (file_entry)) == TRUE) {
 		g_set_error_literal (&error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED,
-				     _("The entry has already been inserted."));
+		                     _("The entry has already been inserted."));
 		goto error;
 	}
 
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (&error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must be authenticated to upload a file."));
+		                     _("You must be authenticated to upload a file."));
 		goto error;
 	}
 
@@ -576,14 +632,14 @@ gdata_picasaweb_service_upload_file_async (GDataPicasaWebService *self, GDataPic
 
 	/* Actually transfer the data */
 	g_output_stream_splice_async (output_stream, input_stream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
-				      0, cancellable, (GAsyncReadyCallback) upload_file_async_cb, data);
+	                              0, cancellable, (GAsyncReadyCallback) upload_file_async_cb, data);
 
 	g_object_unref (input_stream);
 	g_object_unref (output_stream);
 
 	return;
 
- error:
+error:
 	result = g_simple_async_result_new_from_error (G_OBJECT (self), callback, user_data, error);
 	g_simple_async_result_complete (result);
 }
@@ -598,7 +654,7 @@ gdata_picasaweb_service_upload_file_async (GDataPicasaWebService *self, GDataPic
  * Inserts a new album described by @album. A user must be
  * authenticated to use this function.
  *
- * Return value: the inserted #GDataPicasaWebAlbum; unref with
+ * Return value: (transfer full): the inserted #GDataPicasaWebAlbum; unref with
  * g_object_unref()
  *
  * Since: 0.6.0
@@ -606,8 +662,6 @@ gdata_picasaweb_service_upload_file_async (GDataPicasaWebService *self, GDataPic
 GDataPicasaWebAlbum *
 gdata_picasaweb_service_insert_album (GDataPicasaWebService *self, GDataPicasaWebAlbum *album, GCancellable *cancellable, GError **error)
 {
-	GDataCategory *album_kind;
-
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_SERVICE (self), NULL);
 	g_return_val_if_fail (GDATA_IS_PICASAWEB_ALBUM (album), NULL);
 	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
@@ -615,22 +669,16 @@ gdata_picasaweb_service_insert_album (GDataPicasaWebService *self, GDataPicasaWe
 
 	if (gdata_entry_is_inserted (GDATA_ENTRY (album)) == TRUE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED,
-				     _("The album has already been inserted."));
+		                     _("The album has already been inserted."));
 		return NULL;
 	}
 
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must be authenticated to insert an album."));
+		                     _("You must be authenticated to insert an album."));
 		return NULL;
 	}
 
-	/* PicasaWeb needs to know that this is an album, so we're adding the category when we insert it to make sure it's there.
-	   gdata_entry_add_category() checks if it already exists for us. */
-	album_kind = gdata_category_new ("http://schemas.google.com/photos/2007#album", "http://schemas.google.com/g/2005#kind", NULL);
-	gdata_entry_add_category (GDATA_ENTRY (album), album_kind);
-	g_object_unref (album_kind);
-
 	return GDATA_PICASAWEB_ALBUM (gdata_service_insert_entry (GDATA_SERVICE (self), "http://picasaweb.google.com/data/feed/api/user/default",
 	                                                          GDATA_ENTRY (album), cancellable, error));
 }
